<?php

namespace Drupal\raisely;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\Core\Logger\RfcLogLevel;
use GuzzleHttp\Client as GuzzleClient;

/**
 * Class RaiselyApiManager.
 *
 * @method \Drupal\raisely\Resources\RaiselyApi\Donations donations()
 * @method \Drupal\raisely\Resources\RaiselyApi\Donations campaigns()
 */
class RaiselyApiManager {

  /**
   * Default request timeout.
   */
  const REQUEST_TIMEOUT = 30;

  /**
   * Client instance.
   *
   * @var \GuzzleHttp\Client
   */
  public $client;

  /**
   * Raisely Api Url.
   *
   * @var string
   */
  private $apiUrl;

  /**
   * Raisely Api Token.
   *
   * @var string
   */
  private $accessToken;

  /**
   * Enabled Raisely Api.
   *
   * @var bool
   */
  private $enabled;

  /**
   * Verbose logs.
   *
   * @var bool
   */
  private $verbose;

  /**
   * Constructs a new RaiselyApiManager service instance.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\Logger\LoggerChannelInterface $logger_channel
   *   The logger factory.
   */
  public function __construct(ConfigFactoryInterface $config_factory, LoggerChannelInterface $logger_channel) {
    $this->config = $config_factory->get('raisely.config');
    $this->logger = $logger_channel;

    $this->apiUrl = $this->config->get('raisely_api.api_url');
    // TODO: If token can expired - use login credentials to get new token.
    $this->accessToken = $this->config->get('raisely_api.access_token');
    $this->enabled = $this->config->get('raisely_api.enabled');
    $this->verbose = $this->config->get('raisely_api.verbose');

    if (!$this->enabled) {
      $this->logger->info('Raisely API is disabled.');
      return NULL;
    }

    if (empty($this->apiUrl) || empty($this->accessToken)) {
      $this->logger->error('Missing Raisely API configuration.');
      return NULL;
    }

    // Initialize the Client.
    $this->client = new GuzzleClient(['timeout' => self::REQUEST_TIMEOUT]);
  }

  /**
   * Send the request.
   *
   * @param string $method
   *   The HTTP request verb.
   * @param string $endpoint
   *   The Raisely API endpoint (doesn't include base url).
   * @param array $options
   *   An array of options to send with the request.
   *
   * @return \Drupal\raisely\Response
   *   Response object.
   */
  public function request($method, $endpoint, array $options = []) {
    try {
      $url = $this->apiUrl . $endpoint;

      // Adds necessary headers.
      $options['headers']['accept'] = 'application/json';
      $options['headers']['Content-Type'] = 'application/json';
      $options['headers']['Authorization'] = "Bearer " . $this->accessToken;

      if (!empty($this->verbose)) {
        $this->logger
          ->debug('Raisely API Request: %method %url. Query: <pre>%query</pre>  Data: <pre>%data</pre>', [
            '%method' => $method,
            '%url' => $url,
            '%data' => !empty($options['body']) ? json_encode(json_decode($options['body']), JSON_PRETTY_PRINT) : '',
            '%query' => !empty($options['query']) ? json_encode($options['query'], JSON_PRETTY_PRINT) : '',
          ]);
      }

      $response = $this->client->request($method, $url, $options);

      // Wrap to the custom Response class for useful functions.
      $wrapped_response = new Response($response);

      if (!empty($this->verbose)) {
        $data_for_log = $wrapped_response->toArray();
        $additional_message = 'Data :';
        if (
          is_array($data_for_log)
          && array_key_exists('data', $data_for_log)
          && count($data_for_log['data']) > 10) {
          $data_for_log['data'] = array_slice($data_for_log['data'], 0, 10);
          $additional_message = 'First 10 elements from data:';
        }
        $this->logger
          ->debug('Raisely API Response: %url. Status: %status. Query: <pre>%query</pre>  %additional_message <pre>%data</pre>', [
            '%url' => $url,
            '%status' => $wrapped_response->getStatusCode(),
            '%additional_message' => $additional_message,
            '%data' => json_encode($data_for_log, JSON_PRETTY_PRINT),
            '%query' => !empty($options['query']) ? json_encode($options['query'], JSON_PRETTY_PRINT) : '',
          ]);
      }
      return $wrapped_response;
    }
    catch (\Exception $e) {
      watchdog_exception('raisely', $e, NULL, [], RfcLogLevel::DEBUG);
      throw new \Exception($e->getMessage(), $e->getCode(), $e);
    }
  }

  /**
   * Return an instance of a Resource based on the method called.
   *
   * @param string $name
   *   Name of the method.
   * @param array $arguments
   *   Arguments that will be passed further.
   *
   * @return \Drupal\raisely\Resources\Resource
   *   Resource object.
   */
  public function __call($name, array $arguments = NULL) {
    $resource = 'Drupal\\raisely\\Resources\\RaiselyApi\\' . ucfirst($name);

    return new $resource($this);
  }

}
